/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example.carpool;

import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.Assert;

import org.ow2.bonita.example.AbstractExample;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;


public class Carpool extends AbstractExample {

  private static final Logger LOG = Logger.getLogger(Carpool.class.getName());

  public static final int CANCEL_PLACE_IMMEDIATELY = 1;
  public static final int CANCEL_PLACE_AFTER_3_WAIT = 2;
  public static final int CANCEL_REQUEST = 3;
  public static final int ASSOCIATION = 4;

  private static final long CANCEL_REQUEST_TIME = 5000;
  private static final long WAIT_REQUEST_TIME = 5000;
  private static final long WAIT_ANSWER_TIME = 5000;

  private int mode;

  public Map<String, Object> getVariables() {
    if (mode == CANCEL_REQUEST) {
      return null;
    } else if (mode == CANCEL_PLACE_IMMEDIATELY) {
      Map<String, Object> variables = new TreeMap<String, Object>();
      variables.put("offerTimeout", "yes");
      return variables;
    } else if (mode == CANCEL_PLACE_AFTER_3_WAIT) {
      return null;
    } else if (mode == ASSOCIATION) {
      Map<String, Object> variables = new TreeMap<String, Object>();
      variables.put("requestFound", "yes");
      variables.put("answerFound", "yes");
      return variables;
    }
    throw new BonitaRuntimeException("Unknown mode : " + mode);
  }

  public Carpool(int mode) {
    this.mode = mode;
  }

  @Override
  public String getProcessId() {
    return "carpool";
  }

  @Override
  public void afterInstantiate(ProcessInstanceUUID instanceUUID,
      ProcessDefinitionUUID processUUID,
      RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      QueryDefinitionAPI queryDefinitionAPI) throws BonitaException {
    long processingTime = 10000;
    if (mode == CANCEL_REQUEST) {
      waitForActivityExec("WaitRequest", 2, WAIT_REQUEST_TIME * 3, queryRuntimeAPI, instanceUUID);
      runtimeAPI.setProcessInstanceVariable(instanceUUID, "requestFound", "yes");
      waitForActivityExec("WaitAnswer", 1, WAIT_ANSWER_TIME * 2, queryRuntimeAPI, instanceUUID);
      runtimeAPI.setProcessInstanceVariable(instanceUUID, "answerTimeout", "yes");
      processingTime += CANCEL_REQUEST_TIME;
    } else if (mode == CANCEL_PLACE_AFTER_3_WAIT) {
      waitForActivityExec("WaitRequest", 3, WAIT_REQUEST_TIME * 4, queryRuntimeAPI, instanceUUID);
      runtimeAPI.setProcessInstanceVariable(instanceUUID, "offerTimeout", "yes");
    }
    waitForInstanceEnd(processingTime, instanceUUID, queryRuntimeAPI);
  }

  private void waitForActivityExec(String activityId, int execNb, long maxWaitingTime,
      QueryRuntimeAPI queryRuntimeAPI, ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    long start = System.currentTimeMillis();
    do {
      try {
        Set<ActivityInstance<ActivityBody>> activities = queryRuntimeAPI.getActivityInstances(instanceUUID, activityId);
        if (activities != null && activities.size() == execNb) {
          break;
        }
        sleep(500);
      } catch (ActivityNotFoundException e) {
        //nothing : the activity is not yet executed once
        sleep(500);
      }
    } while ((start + maxWaitingTime) > System.currentTimeMillis());
  }

  private void sleep(long milis) {
    try {
      Thread.sleep(milis);
    } catch (InterruptedException e) {
      throw new BonitaRuntimeException(e);
    }
  }

  private void waitForInstanceEnd(long processingTime, ProcessInstanceUUID instanceUUID,
      QueryRuntimeAPI queryRuntimeAPI) throws BonitaException {
    long deadline = System.currentTimeMillis() + processingTime;

    while (System.currentTimeMillis() < deadline) {
      if (InstanceState.FINISHED.equals(queryRuntimeAPI.getProcessInstance(instanceUUID).getInstanceState())) {
        break;
      }
      try {
        Thread.sleep(1000);
      } catch (Exception e) {
        e.printStackTrace();
        Assert.fail();
      }
    }

    ProcessInstance processInstance = queryRuntimeAPI.getProcessInstance(instanceUUID);
    Assert.assertEquals("Instance is not finished !", InstanceState.FINISHED, processInstance.getInstanceState());
  }

  public static void main(String[] args) throws Exception {
    LOG.severe("Usage: " + Carpool.class + " <bar file> <mode>");
    LOG.severe("Available modes ares : ");
    LOG.severe(" 1 -> CANCEL_PLACE_IMMEDIATELY");
    LOG.severe(" 2 -> CANCEL_PLACE_AFTER_3_WAIT");
    LOG.severe(" 3 -> CANCEL_REQUEST");
    LOG.severe(" 4 -> ASSOCIATION");
    if (args == null || args.length != 2) {
      throw new BonitaRuntimeException("Please read the above usage");
    }
    String barFile = args[0];
    int mode = new Integer(args[1]);

    Carpool carpool = new Carpool(mode);
    Map<String, Object> variables = carpool.getVariables();

    carpool.login();
    carpool.execute(
        AccessorUtil.getAPIAccessor().getRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getQueryRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getManagementAPI(),
        AccessorUtil.getAPIAccessor().getQueryDefinitionAPI(),
        barFile, variables);
    carpool.logout();

    if (LOG.isLoggable(Level.INFO)) {
      LOG.info("\n***** End of Carpool Main. *****\n");
    }
  }

}


